dnl @synopsis AC_CHECK_PACKAGE(PACKAGE, FUNCTION, LIBRARY, HEADERFILE [, ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]])
dnl
dnl Provides --with-PACKAGE, --with-PACKAGE-include and --with-PACKAGE-libdir
dnl options to configure. Supports the now standard --with-PACKAGE=DIR
dnl approach where the package's include dir and lib dir are underneath DIR,
dnl but also allows the include and lib directories to be specified seperately
dnl
dnl adds the extra -Ipath to CPPFLAGS if needed
dnl adds extra -Lpath to LD_FLAGS if needed
dnl searches for the FUNCTION in the LIBRARY with
dnl AC_CHECK_LIBRARY and thus adds the lib to LIBS
dnl
dnl defines HAVE_PKG_PACKAGE if it is found, (where PACKAGE in the
dnl HAVE_PKG_PACKAGE is replaced with the actual first parameter passed)
dnl note that autoheader will complain of not having the HAVE_PKG_PACKAGE and you
dnl will have to add it to acconfig.h manually
dnl
dnl @version $Id: aclocal.m4,v 1.1.1.1 2004/01/27 16:22:52 magnard Exp $
dnl @author Paul_Kunz@slac.stanford.edu modified ac_caolan_check_package.m4 for all languages
dnl
AC_DEFUN([AC_CHECK_PACKAGE],
[

AC_ARG_WITH($1,
[  --with-$1[=DIR]      root directory of $1 installation],
with_$1=$withval
if test "${with_$1}" != yes; then
        $1_include="$withval/include"
        $1_libdir="$withval/lib"
fi,
$1_include="/usr/local/cfitsio/include"
$1_libdir="/usr/local/cfitsio/lib"
)

AC_ARG_WITH($1-include,
[  --with-$1-include=DIR        specify exact include dir for $1 headers],
$1_include="$withval")

AC_ARG_WITH($1-libdir,
[  --with-$1-libdir=DIR        specify exact library dir for $1 library
  --without-$1        disables $1 usage completely],
$1_libdir="$withval")

if test "${with_$1}" != no ; then
        OLD_LIBS=$LIBS
        OLD_LDFLAGS=$LDFLAGS
        OLD_CPPFLAGS=$CPPFLAGS

        if test "${$1_libdir}" ; then
                LDFLAGS="$LDFLAGS -L${$1_libdir}"
        fi
        if test "${$1_include}" ; then
                CPPFLAGS="$CPPFLAGS -I${$1_include}"
        fi

        AC_CHECK_LIB($3,$2,,no_good=yes)
        AC_CHECK_HEADER($4,,no_good=yes)
        if test "$no_good" = yes; then
dnl     broken
                ifelse([$6], , , [$6])

                LIBS=$OLD_LIBS
                LDFLAGS=$OLD_LDFLAGS
                CPPFLAGS=$OLD_CPPFLAGS
        else
dnl     fixed
                ifelse([$5], , , [$5])

                AC_DEFINE(HAVE_PKG_$1, 1, Define if you have the package )
        fi

fi

])



AC_DEFUN([AC_LOCAL_LIB],
[
cwd=`pwd`
[cd $srcdir/lib && tar xvzf cfitsio2470.tar.gz && cd cfitsio && ./configure && make && cd $cwd]
AC_SUBST(cfitsio_include, "lib/cfitsio")
AC_SUBST(cfitsio_libdir, "lib/cfitsio")
AC_MSG_WARN(["Using local cfitsio library"])
])


dnl Copyright (C) 2000-2003 Open Source Telecom Corporation.
dnl
dnl This program is free software; you can redistribute it and/or modify
dnl it under the terms of the GNU General Public License as published by
dnl the Free Software Foundation; either version 2 of the License, or
dnl (at your option) any later version.
dnl 
dnl This program is distributed in the hope that it will be useful,
dnl but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl GNU General Public License for more details.
dnl 
dnl You should have received a copy of the GNU General Public License
dnl along with this program; if not, write to the Free Software 
dnl Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
dnl 
dnl As a special exception to the GNU General Public License, if you 
dnl distribute this file as part of a program that contains a configuration 
dnl script generated by Autoconf, you may include it under the same 
dnl distribution terms that you use for the rest of that program.

dnl OST_CCXX2_VERSION([MINIMUM-VERSION[,ACTION-IF-FOUND[,ACTION-IF-NOT-FOUND]]])
dnl Test for usable version of CommonC++

AC_DEFUN(OST_CCXX2_DYNLOADER,[
	ost_cv_dynloader=`$CCGNU2_CONFIG --dso`
	if test "$ost_cv_dynloader" = "yes" ; then
		MODULE_FLAGS=`$CCGNU2_CONFIG --module`
		AC_SUBST(MODULE_FLAGS)
	fi
])

AC_DEFUN(OST_CCXX2_VERSION,
[
  AC_PATH_PROG(CCGNU2_CONFIG, ccgnu2-config, no)
  ccgnu2_version=ifelse([$1], ,0.99.0,$1)
  AC_MSG_CHECKING(for commoncpp2 version >= $ccgnu2_version)
  if test "$CCGNU2_CONFIG" = "no" ; then
    AC_MSG_RESULT(not found)
    echo "*** The ccgnu2-config script installed by commoncpp2 0.99"
    echo "*** or later could not be found."
    echo "*** You need to install GNU Common C++ 2, whose later releases are"
    echo "*** available from http://www.gnu.org/software/commoncpp/ and any"
    echo "*** GNU mirror."
    ifelse([$3], , :, [$3])
    exit -1
  else
    config_version=`$CCGNU2_CONFIG --version`
    ccgnu2_config_major_version=`echo $config_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    ccgnu2_config_minor_version=`echo $config_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    ccgnu2_config_micro_version=`echo $config_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
           
    ccgnu2_check_major_version=`echo "$ccgnu2_version" | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    ccgnu2_check_minor_version=`echo "$ccgnu2_version" | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    ccgnu2_check_micro_version=`echo "$ccgnu2_version" | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
       
    version_ok=no       
    if test $ccgnu2_config_major_version -gt $ccgnu2_check_major_version ; then
	version_ok=yes
    elif test $ccgnu2_config_major_version -eq $ccgnu2_check_major_version  \
       && test $ccgnu2_config_minor_version -gt $ccgnu2_check_minor_version ; then
	version_ok=yes
    elif test $ccgnu2_config_major_version -eq $ccgnu2_check_major_version \
       && test $ccgnu2_config_minor_version -eq $ccgnu2_check_minor_version \
       && test $ccgnu2_config_micro_version -ge $ccgnu2_check_micro_version; then
       
       version_ok=yes
    fi
  
    if test "$version_ok" = "no"; then
      AC_MSG_RESULT(no)
      ost_cv_ccxx_config=false
      echo "*** An old version of CommonC++ of $config_version was found."
      echo "*** You need a version of commoncpp2 newer than $ccgnu2_version. The latest version of"
      echo "*** CommonC++ is always available from ftp://ftp.gnu.org/gnu/commonc++/."
      ifelse([$3], , :, [$3])
    else
      AC_MSG_RESULT(yes)
      ost_cv_ccxx_config=true
      SINGLE_FLAGS="$CXXFLAGS"
      SINGLE_LIBS="$LIBS"
      AC_SUBST(SINGLE_LIBS)
      AC_SUBST(SINGLE_FLAGS)
      CXXFLAGS="$CXXFLAGS "`$CCGNU2_CONFIG --flags`
      GNULIBS="$LIBS "`$CCGNU2_CONFIG --gnulibs`
      EXTLIBS=`$CCGNU2_CONFIG --extlibs`
      LIBS="$LIBS `$CCGNU2_CONFIG --stdlibs`"
      AC_SUBST(GNULIBS)
      AC_SUBST(EXTLIBS)
      CCGNU2_INCLUDES="`$CCGNU2_CONFIG --includes`"
      AC_SUBST(CCGNU2_INCLUDES)
    fi
  fi
])

