///////////////////////////////////////////////////////////////////////////////
// Project: defectix 
// Filename: convert.cc
// Directory: src
///////////////////////////////////////////////////////////////////////////////
// Authors: A. BAILLARD
// Creation date: 16/03/04
// Last modification date: 22/10/04
///////////////////////////////////////////////////////////////////////////////
// Comments:
// 16/03/04	Create from defectix.hh
//		Contain functions to convert from matrix to table an vice versa
//			getMatrixfromImage()
//			template getMatrixfromImage()
//			getImagefromMatrix()
//			template getImagefromMatrix()
//			getPixel()
// 06/04/04	Modify functions to convert from matrices to matrices
//		instead of tables to matrices and matrices to tables
// 16/06/04	Add getVectorfromBloc()
// 17/06/04	Remove getImagefromMatrix() and getMatrixfromImage()
// 30/06/04	Remove getPixel()
// 22/10/04	Add setBlocfromVector()
//		Change class fron Defectix to DefectPCA
///////////////////////////////////////////////////////////////////////////////

#include "pca.hh"

/////////////////////////////////////////////////////////////////////////////
// Image to matrix and matrix to image management methods		   //
/////////////////////////////////////////////////////////////////////////////
void	DefectPCA::getVectorfromBloc(gsl_vector_float *dest, const gsl_matrix_float * const src,
				    unsigned row, unsigned col)
{
  for (int cnt_j = 0; cnt_j < _blocSize; cnt_j++)
    for (int cnt_i = 0; cnt_i < _blocSize; cnt_i++)
      {
	if ((row + cnt_j < src->size1) && (col + cnt_i < src->size2))
	  gsl_vector_float_set(dest, cnt_j * _blocSize + cnt_i, 
			 gsl_matrix_float_get(src, row + cnt_j, col + cnt_i));
	else
	  gsl_vector_float_set(dest, cnt_j * _blocSize + cnt_i, 0);
      }
}

/////////////////////////////////////////////////////////////////////////////
void	DefectPCA::setBlocfromVector(gsl_matrix_float * dest, unsigned row, unsigned col, 
				    const gsl_vector_float * const src)
{
  for (int cnt_j = 0; cnt_j < _blocSize; cnt_j++)
    for (int cnt_i = 0; cnt_i < _blocSize; cnt_i++)
      {
	if ((row + cnt_j < dest->size1) && (col + cnt_i < dest->size2))
	  gsl_matrix_float_set(dest, row + cnt_j, col + cnt_i, 
			 gsl_vector_float_get(src, cnt_j * _blocSize + cnt_i));
      }
}
