///////////////////////////////////////////////////////////////////////////////
// Project: defectix 
// Filename: defectix.cc
// Directory: src
///////////////////////////////////////////////////////////////////////////////
// Authors: A. BAILLARD
// Creation date: 16/03/04
// Last modification date: 04/04/05
///////////////////////////////////////////////////////////////////////////////
// Comments:
// 16/03/04	Create from defectix.hh
//		Contain main functions 
//			constructor and destructor
//			exec()
// 04/05/05	Modify constructor to handle mask mode according to defectix
//		mode via READMODE() macro
///////////////////////////////////////////////////////////////////////////////

#include "defectix.hh"

Defectix::Defectix(const DefectixOptions::Options& options) : 
  ErrorMgr(options.verbose),
  _imageMgr(options.inputFile, READONLY, options.verbose),
  _maskMgr(options.maskFile, READMODE(options.mode), options.verbose),
  _outputMgr(options.outputFile, READWRITE, options.verbose)
{
  if ((setError("Defectix::Defectix", _imageMgr.getError())) == NO_ERROR &&
      (setError("Defectix::Defectix", _maskMgr.getError())) == NO_ERROR &&
      (setError("Defectix::Defectix", _outputMgr.getError())) == NO_ERROR) 
    {
      time_t	ttime;
      struct tm	*ttm;
	
      // init random seed with current time
      time(&ttime);
      ttm = gmtime(&ttime);
      srand(ttm->tm_sec + ttm->tm_min + ttm->tm_hour);

      _PCA = NULL;
      _factor = options.rebinFactor;
      _blocSize = options.blocSize;
      _substractBg = options.substractBg;
      _reduceDyn = options.reduceDyn;
      _zoomStep = options.zoomStep;
      _imageMgr.pHDU();
    }
}
