///////////////////////////////////////////////////////////////////////////////
// Project: defectix 
// Filename: defectix.hh
// Directory: src
///////////////////////////////////////////////////////////////////////////////
// Authors: A. BAILLARD
// Creation date: 16/03/04
// Last modification date: 16/03/04
///////////////////////////////////////////////////////////////////////////////
// Comments:
// 16/03/04	Create from defoptions.hh
///////////////////////////////////////////////////////////////////////////////

#include <string>
#include <iostream>
#include "common.hh"

static struct DefectixOptions::sOptions<DefectixOptions::mode> modeNames[] =
  {
    {DefectixOptions::HALO, "halo"},
    {DefectixOptions::TRAIL, "trail"},
    {DefectixOptions::TRAINING, "training"},
    {DefectixOptions::COMPUTING, "computing"},
    {DefectixOptions::NOMODE, NULL}
  };

static struct DefectixOptions::sOptions<DefectixOptions::simulatortype> simulatorTypeNames[] =
  {
    {DefectixOptions::EXTENSION, "extension"},
    {DefectixOptions::MOSAIC, "mosaic"},
    {DefectixOptions::NOTYPE, NULL}
  };

void DefectixOptions::usage(void)
{
  std::cout << "\
Usage: defectix -M <mode> {-C <name> | [options]} \n\
\n\
  -M, --mode <mode>		specify the defectix mode to use :\n\
				  halo\n\
				  trail\n\
				  training\n\
				  computing\n\
\n\
  -C, --conf-file <name>	specify the configuration file to load\n\
				when loaded, command line options are ignored\n\
  -I, --input-file <name>	specify the MEF file to treat\n\
  -K, --mask-file <name>	specify the MEF file for masks\n\
  -O, --output-file <name>	specify the MEF file for output (if needed)\n\
  -L, --evec-file <name>	specify the gsl_matrix_float for the KL-base\n\
  -N, --nn-file <name>		specify the neural network\n\
\n\
Options:\n\
  -d, --defaults		display default options and exit\n\
  -h, --help			display this help and exit\n\
  -V, --version			display the version and exit\n\
  -v, --verbose			display error messages\n\
\n\
  -S, --substract-bg <bool>	ask for a background substraction on image\n\
  -D, --reduce-dynamic <bool>	ask for a reduction of the image dynamic range\n\
  -r, --rebin-factor <value>	specify a rebin factor for image loading\n\
  -b, --bloc-size <value>	specify a size for PCA blocs\n\
\n\
For 'trainer' and 'computer' mode only\n\
  -p, --nb-pc <value>		specify the number of components to use\n\
  -t, --nb-vector <value>	specicy the number of vectors used for training\n\
\n\
  -e, --energy-treshold <value>	define a treshold for neural network convergence\n\
  -m, --max-iteration <value>	define a maximum number of iterations for\n\
				neural network convergence\n\
  -x, --max-prior <value>	define a maximum intensity value for priors\n\
  -n, --min-prior <value>	define a minimum intensity value for priors\n\
  -l, --non-prior <value>	define a part of blocs to use without priors\n\
\n\
  -s, --mask-treshold <value>	define a treshold for mask creation\n\
\n\
  -E, --print-evec		print eigen vectors of the PCA into files\n\
  -R, --print-rebuilt		print the rebuilt image by the PCA into a file\n\
\n\
For simulating modes only\n\
  -a, --adder-type <type>	specify the defect adder type :\n\
				  extension (nb-defect per extension)\n\
				  mosaic (nb-defect on the whole mosaic)\n\
  -i, --intensity <value>	specify the defect intensity as a divisor\n\
  -f, --nb-defect <value>	specify the number of defects to add"
	    << std::endl;
  
  exit(0);
}
  
void DefectixOptions::version(void)
{
  std::cout << MESSAGE_START << "(" << PACKAGE_STRING << ") " << CDATE << std::endl;
  std::cout << "Written by Anthony Baillard (baillard@iap.fr)" << std::endl;
  
  exit(0);
}

void DefectixOptions::defaultconf(int mode)
{
  std::cout << "\
## configuration file for defectix\n\
## use 'defectix -C' or 'defectix --conf-file'\n\
## /!\\ configuration file options cancel command line options\n\
## add # at the begin of a line to comment it\n\
\n\
\n\
###################### following options are always used ######################\n\
## execution mode ('halo', 'trail, 'training' or 'computing')\n\
MODE trail\n\
\n\
## input file (no default, must be chosen)\n\
## do not forget '!' to create or overwrite a file\n\
INPUT-FILE 715230p.fits\n\
\n\
# /!\\ OUTPUT-FILE is usefull to see PCA result in training mode\n\
OUTPUT-FILE !rebuilt.sim.fits\n\
\n\
# /!\\ MASK-FILE is read for training and written for computing or simulating\n\
MASK-FILE !mask.sim.fits\n\
\n\
# /!\\ EVEC-FILE is read for computing and written for training\n\
EVEC-FILE evec.mat.gsl\n\
\n\
# /!\\ NN-FILE is read for computing and written for training\n\
NN-FILE network.net\n\
\n\
## size of blocs for the PCA (default 8)\n\
BLOC-SIZE 8\n\
\n\
## rebin factor (default is 4)\n\
#REBIN-FACTOR 8\n\
\n\
## ask for substraction of the background (default true)\n\
#SUBSTRACT-BG false\n\
\n\
## reduce dynamic range of the image (default true)\n\
#REDUCE-DYN false\n\
\n\
############## following options are used for 'halo' or 'trail' mode ##########\n\
## defect adder type \n\
## extension = NB-DEFECTS per extension\n\
## mosaic = NB-DEFECTS on the whole mosaic\n\
ADDER-TYPE extension\n\
\n\
# intensity of the defect compared to maximum (1 / INTENSITY) (default 200)\n\
INTENSITY 8\n\
\n\
## number of defects to add corresponding to the defect adder type\n\
NB-DEFECT 10\n\
\n\
\n\
############### following options are use for 'training' mode #################\n\
## output results of the PCA in files (default does not output)\n\
#PRINT-REBUILT\n\
PRINT-EVEC\n\
\n\
## number of principal components to use after PCA (default 500000)\n\
NB-VECTOR 400000\n\
\n\
## number of principal components to use after PCA (default 32)\n\
NB-PC 8\n\
\n\
## energy treshold for training (default 0.025)\n\
ENERGY-TRESHOLD 0.001\n\
\n\
## maximum number of iterations with the same error value for training \n\
## (default 200)\n\
MAX-ITERATION 20\n\
\n\
## prior management for training\n\
MAX-PRIOR 1\n\
MIN-PRIOR 0.3\n\
NON-PRIOR 0.5\n\
\n\
\n\
############### following options are use for 'computing' mode #################\n\
## number of principal components to use after PCA (default 32)\n\
NB-PC 8\n\
\n\
## energy treshold for training (default 1 = no treshold)\n\
MASK-TRESHOLD 0.5\n\
\n\
## end of file\n" << std::endl;
  
  exit(0);
}
  

bool DefectixOptions::set_file_option(char *line, char *keyword, int key_opt, Options &options)
{
  size_t	key_length;
  size_t	line_length;
  
  line_length = strlen(line);
  key_length = strlen(keyword);
  
  for (size_t n = 0; n <= key_length; n++)
    line[n] = tolower(line[n]);
  
  if (!strncmp(&line[1], keyword, key_length))
    {
      if (key_length + 2 < line_length)
	optarg = &line[key_length + 2];
      set_option(key_opt, options);
      return true;
    }
  return false;
}

int	DefectixOptions::conf_file_error(int line)
{
  std::cerr << WARNING_START 
	    << "syntax error in configuration file (line " 
	    << line << ")" << std::endl;
  
  return 1;
}

void	DefectixOptions::read_conf_file(FILE *f, Options& options)
{
  char		*line = NULL;
  size_t	length;
  int		cnt;
  
  cnt = 1;
  while (getline(&line, &length, f) != -1)
    {
      line[0] = tolower(line[0]);
      line[strlen(line) - 1] = '\0';
      
      switch (line[0])
	{
	case '#': case '\0': break;
	
	case 'a':
	  {
	    set_file_option(line, "dder-type\0", opt_addertype, options) ||
	      conf_file_error(cnt);
	    break;
	  }
  
	case 'b':
	  {
	    set_file_option(line, "loc-size\0", opt_bsize, options) ||
	      conf_file_error(cnt);
	    break;
	  }
	    
	case 'e':
	  {
	    set_file_option(line, "vec-file\0", opt_efile, options) ||
	      set_file_option(line, "nergy-treshold\0", opt_energy, options) ||
	      conf_file_error(cnt);
	    break;
	  }

	case 'i':
	  {
	    set_file_option(line, "nput-file\0", opt_input, options) ||
	    set_file_option(line, "ntensity\0", opt_intensity, options) ||
	      conf_file_error(cnt);
	    break;
	  }
	    
	case 'm':
	  {
	    set_file_option(line, "ode\0", opt_mode, options) ||
	      set_file_option(line, "ask-file\0", opt_mask, options) ||
	      set_file_option(line, "ax-iteration\0", opt_maxiter, options) ||
	      set_file_option(line, "ax-prior\0", opt_maxprior, options) ||
	      set_file_option(line, "in-prior\0", opt_minprior, options) ||
	      set_file_option(line, "ask-treshold\0", opt_mtreshold, options) ||
	      conf_file_error(cnt);
	    break;
	  }
	  
	case 'n':
	  {
	    set_file_option(line, "b-pc\0", opt_nbpc, options) ||
	      set_file_option(line, "n-file\0", opt_nnfile, options) ||
	      set_file_option(line, "b-vector\0", opt_nbvec, options) ||
	      set_file_option(line, "on-prior\0", opt_nonprior, options) ||
	      set_file_option(line, "b-defect\0", opt_nbdef, options) ||
	      conf_file_error(cnt);
	    break;
	  }
	  
	case 'o':
	  {
	    set_file_option(line, "utput-file\0", opt_output, options) ||
	      conf_file_error(cnt);
	    break;
	  }
	  
	case 's':
	  {
	    set_file_option(line, "ubstract-bg\0", opt_subbg, options) ||
	      conf_file_error(cnt);
	    break;
	  }
	  
	case 'p':
	  {
	    set_file_option(line, "rint-rebuilt\0", opt_rebuilt, options) ||
	      set_file_option(line, "rint-evec\0", opt_evec, options) ||
	      conf_file_error(cnt);
	    break;
	  }
	  
	case 'r':
	  {
	    set_file_option(line, "ebin-factor\0", opt_rebin, options) ||
	      set_file_option(line, "educe-dyn\0", opt_dyn, options) ||
	      conf_file_error(cnt);
	    break;
	  }

	case 'v':
	  {
	    set_file_option(line, "erbose\0", opt_verbose, options) ||
	      conf_file_error(cnt);
	    break;
	  }
	  
	case 'z':
	  {
	    set_file_option(line, "oom-step\0", opt_zoom, options) ||
	      conf_file_error(cnt);
	    break;
	  }
	  
	default:
	  {
	    conf_file_error(cnt);
	    break;
	  }
	}
      cnt++;
    } 
}

void	DefectixOptions::set_option(int c, Options& options)
{
  if (!options.confFile.empty())
    return;
  
  switch (c)
    {
    case opt_input:
      {
	options.inputFile = optarg;
	break;
      }
      
    case opt_mask:
      {
	options.maskFile = optarg;
	break;
      }
      
    case opt_output:
      {
	options.outputFile = optarg;
	break;
      }

    case opt_efile:
      {
	options.evecFile = optarg;
	break;
      }

    case opt_nnfile:
      {
	options.nnFile = optarg;
	break;
      }

    case opt_conf:
      {
	FILE	*f;
	options.reinit();
	
	if ((f = fopen(optarg, "r")) == NULL)
	  std::cerr << WARNING_START << "wrong configuration file, "
		    << "using defaults values" << std::endl;
	else
	  read_conf_file(f, options);
	
	options.confFile = optarg;
	break;
      }
      
    case opt_default: defaultconf(options.mode);

    case opt_help: usage();
      
    case opt_version: version();
      
    case opt_mode:
      {
	unsigned int cnt;
	
	for (cnt = 0; cnt < strlen(optarg); cnt++)
	  optarg[cnt] = tolower(optarg[cnt]);
	
	for (cnt = 0; modeNames[cnt].name != NULL; cnt++)
	  if (!strcmp(optarg, modeNames[cnt].name))
	    {
	      options.mode = modeNames[cnt].value;
	      break;
	    }
	
	if (modeNames[cnt].name == NULL)
	    std::cerr << WARNING_START << "invalid value for M"
		      << std::endl;
	break;
      }
      
    case opt_rebin:
      {
	int tmp = atoi(optarg);
	if (tmp > 0)
	  options.rebinFactor = tmp;
	else
	  std::cerr << WARNING_START << "invalid value for r, "
		    << "using default value."  << std::endl;
	
	break;
      }
      
    case opt_bsize:
      {
	int tmp = atoi(optarg);
	if (tmp > 0)
	  options.blocSize = tmp;
	else
	  std::cerr << WARNING_START << "invalid value for b, "
		    << "using default value."  << std::endl;
	
	break;
      }
      
    case opt_nbpc:
      {
	int tmp = atoi(optarg);
	if (tmp > 0)
	  options.nbPC = tmp;
	else
	  std::cerr << WARNING_START << "invalid value for p, "
		    << "using default value."  << std::endl;
	break;
      }

    case opt_nbvec:
      {
	int tmp = atoi(optarg);
	if (tmp > 0)
	  options.nbVectors = tmp;
	else
	  std::cerr << WARNING_START << "invalid value for t, "
		    << "using default value."  << std::endl;
	break;
      }
	
    case opt_subbg:
      {
	if (!strncmp(optarg, "true", 4))
	  options.substractBg = true;
	else if (!strncmp(optarg, "false", 5))
	  options.substractBg = false;
	else
	  std::cerr << WARNING_START << "invalid value for S, "
		    << "using default value."  << std::endl;
	break;
      }
      
    case opt_dyn:
      {
	if (!strncmp(optarg, "true", 4))
	  options.reduceDyn = true;
	else if (!strncmp(optarg, "false", 5))
	  options.reduceDyn = false;
	else
	  std::cerr << WARNING_START << "invalid value for D, "
		    << "using default value."  << std::endl;
	break;
      }
      
    case opt_evec:
      {
	options.printEvec = true;
	break;
      }
      
    case opt_rebuilt:
      {
	options.printRebuilt = true;
	break;
      }
      
    case opt_nbdef:
      {
	int tmp = atoi(optarg);
	if (tmp >= 0)
	  options.nbDefects = tmp;
	else
	  std::cerr << WARNING_START << "invalid value for d, "
		    << "using default value."  << std::endl;
	break;
      }
      
    case opt_zoom:
      {
	int tmp = atoi(optarg);
	if (tmp > 0)
	  options.zoomStep = tmp;
	else
	  std::cerr << WARNING_START << "invalid value for z, "
		    << "using default value."  << std::endl;
	break;
      } 
      
    case opt_addertype:
      {
	unsigned int cnt;
	
	for (cnt = 0; cnt < strlen(optarg); cnt++)
	  optarg[cnt] = tolower(optarg[cnt]);
	
	for (cnt = 0; simulatorTypeNames[cnt].name != NULL; cnt++)
	  if (!strcmp(optarg, simulatorTypeNames[cnt].name))
	    {
	      options.simulatorType = simulatorTypeNames[cnt].value;
	      break;
	    }
	
	if (simulatorTypeNames[cnt].name == NULL)
	  std::cerr << WARNING_START << "invalid value for a, "
		    << "using default value."  << std::endl;
	break;
      }

    case opt_verbose:
      {
	options.verbose = true;
	break;
      }

    case opt_intensity:
      {
	int tmp = atoi(optarg);
	if (tmp > 0)
	  options.intensity = tmp;
	else
	  std::cerr << WARNING_START << "invalid value for i, "
		    << "using default value."  << std::endl;
	break;
      }

    case opt_maxiter:
      {
	int tmp = atoi(optarg);
	if (tmp > 0)
	  options.maxIter = tmp;
	else
	  std::cerr << WARNING_START << "invalid value for m, "
		    << "using default value."  << std::endl;
	break;
      }

    case opt_energy:
      {
	double tmp = atof(optarg);
	if (tmp > 0)
	  options.energyTreshold = tmp;
	else
	  std::cerr << WARNING_START << "invalid value for e, "
		    << "using default value."  << std::endl;
	break;
      }

    case opt_mtreshold:
      {
	double tmp = atof(optarg);
	if (tmp > 0)
	  options.maskTreshold = tmp;
	else
	  std::cerr << WARNING_START << "invalid value for s, "
		    << "using default value."  << std::endl;
	break;
      }

    case opt_maxprior:
      {
	double tmp = atof(optarg);
	options.maxPrior = tmp;
	break;
      }

    case opt_minprior:
      {
	double tmp = atof(optarg);
	options.minPrior = tmp;
	break;
      }

    case opt_nonprior:
      {
	double tmp = atof(optarg);
	if (tmp > 0 && tmp < 1)
	  options.nonPrior = tmp;
	else
	  std::cerr << WARNING_START << "invalid value for l, "
		    << "using default value."  << std::endl;

	break;
      }
    }
} 
