///////////////////////////////////////////////////////////////////////////////
// Project: defectix 
// Filename: fitsread.cc
// Directory: src
///////////////////////////////////////////////////////////////////////////////
// Authors: A. BAILLARD
// Creation date: 18/03/04
// Last modification date: 18/03/04
///////////////////////////////////////////////////////////////////////////////
// Comments:
// 18/03/04	Move code from fitsmgr.hh
//		Contain reading methods
//			readImage()
//			unreadImage()
///////////////////////////////////////////////////////////////////////////////

# include "fitsmgr.hh"

void	FitsManager::readImage(void **img)
{
  if ((_currentExt > 0 && _currentType == IMAGE_HDU) || _type == NO_EXTENDED)
    {
      int status(0);
      int nElements(1);

      for (int cnt = 0; cnt < _naxis; cnt++)
	nElements *= _naxes[cnt];

      if (_bitpix == FLOAT_IMG)
	{
	  *img = new (float[nElements]);
	  fits_read_pix(_file, TFLOAT, _fpixel, nElements, NULL, 
			*img, NULL, &status);
	}
      else if (_bitpix == DOUBLE_IMG)
	{
	  *img = new (double[nElements]);
	  fits_read_pix(_file, TDOUBLE, _fpixel, nElements, NULL, 
			*img, NULL, &status);
	}
      else if (_bitpix == BYTE_IMG)
	{
	  *img = new (char[nElements]);
	  fits_read_pix(_file, TBYTE, _fpixel, nElements, NULL, 
			*img, NULL, &status);
	} 
      else if (_bitpix == LONG_IMG)
	{
	  *img = new (long[nElements]);
	  fits_read_pix(_file, TLONG, _fpixel, nElements, NULL, 
			*img, NULL, &status);
	}    
      else if (_bitpix == SHORT_IMG)
	{
	  *img = new (int[nElements]);
	  fits_read_pix(_file, TINT, _fpixel, nElements, NULL, 
			*img, NULL, &status);
	}
      setFitsError(status);
    }
  else
    _error = DefectixError::NOT_IMGEXT;
}
  
void	FitsManager::unreadImage(void **img)
{
  if (_bitpix == FLOAT_IMG)
    delete (float *)*img;
    
  else if (_bitpix == DOUBLE_IMG)
    delete (double *)*img;
    
  else if (_bitpix == BYTE_IMG)
    delete (char *)*img;
    
  else if (_bitpix == LONG_IMG)
    delete (long *)*img;
    
  else if (_bitpix == SHORT_IMG)
    delete (int *)*img;
    
  *img = NULL;
}
