///////////////////////////////////////////////////////////////////////////////
// Project: defectix 
// Filename: fitswrite.cc
// Directory: src
///////////////////////////////////////////////////////////////////////////////
// Authors: A. BAILLARD
// Creation date: 18/03/04
// Last modification date: 13/05/05
///////////////////////////////////////////////////////////////////////////////
// Comments:
// 18/03/04	Move code from fitsmgr.hh
//		Contain write methods
//			setHDUAsImage()
//			writeImage()
// 13/05/05	Change setHDUAsImage() to update keyword NAXIS*
//		/!\ WARNING: should not work if number of axes differ
//		Change writeImage() to remove fits_create_img()
//		These modifications are coherent with those did in write.cc
///////////////////////////////////////////////////////////////////////////////

# include "fitsmgr.hh"

void	FitsManager::setHDUAsImage(int bitpix, 
				   const long naxis, 
				   const long *axes, 
				   const long factor)
{
  char		str[8];
  int		status;

  _currentType = IMAGE_HDU;
  _bitpix = bitpix;
    
  if (naxis != _naxis)
    {
      _naxis = naxis;
      delete (_naxes);
      delete (_fpixel);
      _naxes = new long(_naxis);
      _fpixel = new long(_naxis);	
    }
  for (int cnt = 0; cnt < _naxis; cnt++)
    {
      _fpixel[cnt] = 1;
      _naxes[cnt] = axes[cnt] / factor;
      sprintf(str, "NAXIS%d", cnt + 1);
      fits_modify_key_lng(_file, str, _naxes[cnt], "&", &status);
    }
  if (status)
    setFitsError(status);
}
  
void	FitsManager::writeImage(void **img)
{
  int status(0);
  int nElements(1);
    
  for (int cnt = 0; cnt < _naxis; cnt++)
    nElements *= _naxes[cnt];
    
  if (_bitpix == FLOAT_IMG)
    {
      fits_write_pix(_file, TFLOAT, _fpixel, nElements, 
		     *img, &status);
    }
  else if (_bitpix == DOUBLE_IMG)
    {
      fits_write_pix(_file, TDOUBLE, _fpixel, nElements, 
		     *img, &status);
    }
  else if (_bitpix == BYTE_IMG)
    {
      fits_write_pix(_file, TBYTE, _fpixel, nElements, 
		     *img, &status);
    } 
  else if (_bitpix == LONG_IMG)
    {
      fits_write_pix(_file, TLONG, _fpixel, nElements, 
		     *img, &status);
    }    
  else if (_bitpix == SHORT_IMG)
    {
      fits_write_pix(_file, TINT, _fpixel, nElements, 
		     *img, &status);
    }
  setFitsError(status);
  _currentExt++;
}
