///////////////////////////////////////////////////////////////////////////////
// Project: defectix 
// Filename: halosimulator.cc
// Directory: src
///////////////////////////////////////////////////////////////////////////////
// Authors: A. BAILLARD
// Creation date: 18/03/04
// Last modification date: 18/03/04
///////////////////////////////////////////////////////////////////////////////
// Comments:
// 15/03/04	Copy code from astrosimulator.hh
// 30/06/04	Update to halosimulator.cc
///////////////////////////////////////////////////////////////////////////////

#include "halosimulator.hh"

using namespace DefectixError;

/////////////////////////////////////////////////////////////////////////////
HaloSimulator::HaloSimulator(const DefectixOptions::Options& options) : 
  DefectSimulator(options)
{
}

// no error can occur in createHalo because everything was checked 
// in addDefectOnExtension()
void		HaloSimulator::createDefect(gsl_matrix_float *src, gsl_matrix_float *mask, int extend)
{
  int		row;
  int		col;
  int		pos[2];
  double	ray;
  double	value;
  double	max;
  int		axis[2];

  axis[0] = _imageMgr.getAxis(0) / _factor;
  axis[1] = _imageMgr.getAxis(1) / _factor;
  pos[0] = (int) (axis[0] * (rand()/(RAND_MAX + 1.0)));
  pos[1] = (int) (axis[1] * (rand()/(RAND_MAX + 1.0)));
  ray = ((axis[0] / 4.0) * (rand()/(RAND_MAX + 1.0)) + (axis[0] / 5.0));
  value = _imagesInfo[extend].max / _intensity;
  max = value + 0.5 * value;
  if (lrint((rand()/(RAND_MAX + 1.0))))
    value += 0.5 * (rand()/(RAND_MAX + 1.0)) * value;
  else
    value -= 0.5 * (rand()/(RAND_MAX + 1.0)) * value;

  for (row = 0; row < axis[1]; ++row)
    for (col = 0; col < axis[0]; ++col)
      {
	double d = sqrt(pow((row - pos[1]), 2) + pow((col - pos[0]), 2));
	if (d <= ray && d > (ray / 4) && 
	    fabs(col - pos[0]) > (ray / 15) && fabs(row - pos[1]) > (ray / 15))
	  {
	    gsl_matrix_float_set(src, row, col, gsl_matrix_float_get(src, row, col) + value);
	    gsl_matrix_float_set(mask, row, col, value / max);
	  }
      }
}
