/******************************************************************************
** Project: neural_network
** Filename: activation_function.c
*******************************************************************************
** Authors: A. BAILLARD & T. BAUDLOT
** Creation date: 01/07/03
** Last modification date: 15/04/04
*******************************************************************************
** Comments:
** 01/07/03	AB	Defines activation function pointers
** 15/04/04	AB	Add an activation function 
**			(sigmoid_0_1 and sigmoid_0_1_drv)
******************************************************************************/

#include <math.h>
#include "activation_function.h"

double		sigmoid(double value, double lambda)
{
  return 2.0 / (1.0 + exp(-lambda * value)) - 1.0;
}

double		sigmoid_drv(double value, double lambda)
{
  double	e = exp(-lambda * value);

  return 2.0 * lambda * ((1 / (1 + e)) - (1 / ((1 + e) * (1 + e))));
}

double		sigmoid_0_1(double value, double lambda)
{
  return 1.0 / (1.0 + exp(-lambda * value));
}

double		sigmoid_0_1_drv(double value, double lambda)
{
  double	e = exp(-lambda * value);

  return lambda * ((1 / (1 + e)) - (1 / ((1 + e) * (1 + e))));
}
