/******************************************************************************
** Project: neural_network
** Filename: activation_function.c
*******************************************************************************
** Authors: A. BAILLARD & T. BAUDLOT
** Creation date: 01/07/03
** Last modification date: 15/04/04
*******************************************************************************
** Comments:
** 01/07/03	AB	Defines enum for available activation functions
** 15/04/04	AB	Add an activation function 
**			(sigmoid_0_1 and sigmoid_0_1_drv)
******************************************************************************/

#ifndef ACFUNC_H_
# define ACFUNC_H_

/**
 ** e_acfunc defines enum values for available functions
 **/
typedef enum	e_acfunc
{
  SIGMOID, SIGMOID_0_1, END
}		e_acfunc;

/**
 ** each function is composed of
 **	id: enum value corresponding to the function name
 **	func: function
 **	func_drv: first degree derivated function
 **	lambda: an optional parameter
 **/
typedef struct 	s_acfunc
{
  char		*name;
  e_acfunc	id;
  double       	(*func)(double, double);
  double       	(*func_drv)(double, double);
  double	lambda;
}		t_acfunc;

/* sigmoid between -1 and 1 */
double		sigmoid(double value, double lambda);
double		sigmoid_drv(double value, double lambda);

/* sigmoid between 0 and 1 */
double		sigmoid_0_1(double value, double lambda);
double		sigmoid_0_1_drv(double value, double lambda);

#endif /* !ACFUNC_H_ */
