/******************************************************************************
** Project: neural_network
** Filename: common.c
*******************************************************************************
** Authors: A. BAILLARD & T. BAUDLOT
** Creation date: 01/07/03
** Last modification date:
*******************************************************************************
** Comments:
** 01/07/03	AB	Defines common functions
**				- xmalloc()
** 14/07/03	AB	Defines functions for files management
**			- xfopen()
**			- xfread()
**			- xfwrite()
******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "common.h"

void	*xmalloc(unsigned size)
{
  void	*p;
  if ((p = malloc(size)) == NULL)
    {
      printf("Error: Cannot allocate enough space\n");
      exit(LIBNN_MALLOC_ERROR);
    }
  
  return p;
}

void	*xfopen(const char *name, const char *flags)
{
  void	*p;

  if ((p = fopen(name, flags)) == NULL)
    {
      printf("Error: Cannot open file '%s'\n", name);
      exit(LIBNN_OPEN_ERROR);
    }

  return p;
}

void	xfread(void *dest, unsigned size, unsigned nb, FILE *file)
{
  if (fread(dest, size, nb, file) != nb)
    {
      printf("Error: Cannot read file\n");
      exit(LIBNN_READ_ERROR);
    }
}

void	xfwrite(void *dest, unsigned size, unsigned nb, FILE *file)
{
  if (fwrite(dest, size, nb, file) != nb)
    {
      printf("Error: Cannot write file\n");
      exit(LIBNN_WRITE_ERROR);
    }
}

double	minimum(double a, double b)
{
  return (a > b) ? b : a;
}

double	maximum(double a, double b)
{
  return (a > b) ? a : b;
}

double	sign(double a)
{
  return (a > 0.) ? 1. : ((a < 0.) ? -1. : 0.);
}

double	xfabs(double a)
{
  return (a > 0) ? a : -a;
}

char		*toupper_string(char *string)
{
  char		*tmp;
  unsigned	cnt;

  tmp = (char *)malloc(strlen(string) + 1);

  for (cnt = 0; cnt <= strlen(string); cnt++)
    {
      tmp[cnt] = toupper(string[cnt]);
    }

  return tmp;
}
