/******************************************************************************
** Project: neural_network
** Filename: file_management.c
*******************************************************************************
** Authors: A. BAILLARD & T. BAUDLOT
** Creation date: 14/07/03
** Last modification date: 14/07/03
*******************************************************************************
** Comments:
** 01/07/03	AB	create functions to save network configuration
**			- load_network_config()
**			- save_network_config()
******************************************************************************/

#include <stdio.h>
#include "network.h"

/**
 **
 **/
t_network	*load_network_config(const char *file_name)
{
  FILE		*file;
  t_network	*network;
  unsigned	acfunc;
  unsigned	nb_input;
  unsigned	nb_hidden;
  unsigned	nb_output;
  unsigned	i;
  unsigned	j;

  file = xfopen(file_name, "r");

  nb_input = nb_hidden = nb_output = 0;
  
  xfread(&nb_input, sizeof(unsigned), 1, file);
  xfread(&nb_hidden, sizeof(unsigned), 1, file);
  xfread(&nb_output, sizeof(unsigned), 1, file);

  printf("%u, %u, %u\n", nb_input, nb_hidden, nb_output);

  xfread(&acfunc, sizeof(unsigned), 1, file);

  network = create_network2(nb_input, nb_hidden, nb_output, acfunc);

  xfread(&(network->energy), sizeof(double), 1, file);
  xfread(&(network->epsilon), sizeof(double), 1, file);

  for (i = 0; i < network->nb_hidden_units; i++)
    {
        xfread(&(network->hidden_biases[i]), sizeof(double), 1, file);
	for (j = 0; j < network->nb_input_units; j++)
	          xfread(&(network->hidden_weights[i][j]), 
			 sizeof(double), 1, file);
    }

  for (i = 0; i < network->nb_output_units; i++)
    {
        xfread(&(network->output_biases[i]), sizeof(double), 1, file);
	for (j = 0; j < network->nb_hidden_units; j++)
	          xfread(&(network->output_weights[i][j]), 
			 sizeof(double), 1, file);
    }

  fclose(file);
  return network;
}

void		save_network_config(t_network *network, const char *file_name)
{
  FILE		*file;
  unsigned	i;
  unsigned	j;
 
  file = xfopen(file_name, "w");

  xfwrite(&network->nb_input_units, sizeof(unsigned), 1, file);
  xfwrite(&network->nb_hidden_units, sizeof(unsigned), 1, file);
  xfwrite(&network->nb_output_units, sizeof(unsigned), 1, file);

  xfwrite(&network->acfunc.id, sizeof(unsigned), 1, file);

  xfwrite(&network->energy, sizeof(double), 1, file);
  xfwrite(&network->epsilon, sizeof(double), 1, file);

  for (i = 0; i < network->nb_hidden_units; i++)
    {
        xfwrite(&(network->hidden_biases[i]), sizeof(double), 1, file);
	for (j = 0; j < network->nb_input_units; j++)
	          xfwrite(&(network->hidden_weights[i][j]), 
			 sizeof(double), 1, file);
    }

  for (i = 0; i < network->nb_output_units; i++)
    {
        xfwrite(&(network->output_biases[i]), sizeof(double), 1, file);
	for (j = 0; j < network->nb_hidden_units; j++)
	          xfwrite(&(network->output_weights[i][j]), 
			 sizeof(double), 1, file);
    }
  
  fclose(file);
}
