/******************************************************************************
** Project: neural_network
** Filename: network.h
*******************************************************************************
** Authors: A. BAILLARD & T. BAUDLOT
** Creation date: 01/07/03
** Last modification date: 11/07/03
*******************************************************************************
** Comments:
** 01/07/03	AB	Creation of network structure
** 04/07/03	AB	Add energy vectors to structure
** 08/07/03	TB	Add Epsilon
** 11/07/03	TB	Add Hidden and Output biases
******************************************************************************/

#ifndef NETWORK_H_
# define NETWORK_H_
/**
 ** versioning and credits messages (for wrapper)
 **/
# define CREDITS_STRING		"\
Thanks to members of bioinformatics unit at Pasteur \
Institute (M. NILGES, W. RIEPING and M. HABECK).\n\
See www.epita.fr/~bailla_a/scia_project for more information.\n"

# define COPYRIGHT_STRING	"\
See GPL license instructions.\n"

# define LICENSE_STRING		"\
Module _nn is under GPL license. See www.gnu.org for \
more information.\n"



/**
 ** include headers needed for network management
 **/
# include <stdio.h>
# include "common.h"
# include "activation_function.h"

# define EPSILON	0.01
# define MAX_ITER	200

typedef struct	s_network
{
  t_acfunc	acfunc;

  double	energy;
  double	epsilon;

  unsigned     	nb_input_units;
  unsigned     	nb_hidden_units;
  unsigned     	nb_output_units;

  double	*hidden_activation;
  double	*hidden_values;
  double	*hidden_biases;

  double	*output_activation;
  double	*output_values;
  double	*output_biases;

  double	**hidden_weights;
  double	**output_weights;

  double	*hidden_energy;
  double	*output_energy;

  unsigned	max_iter;
  double	energy_treshold;

}		t_network;

/**
 ** include headers for additional neural network management
 **/
# include "prop.h"
# include "file_management.h"


/**
 ** Prototypes
 **/ 
t_network      	*create_network(unsigned nb_input_nodes, 
			       unsigned nb_hidden_nodes, 
			       unsigned nb_output_nodes, 
			       char *activation_function);

t_network      	*create_network2(unsigned nb_input_nodes, 
			       unsigned nb_hidden_nodes, 
			       unsigned nb_output_nodes, 
			       int activation_function);

t_network      	*create_network_common(unsigned nb_input_nodes, 
			       unsigned nb_hidden_nodes, 
			       unsigned nb_output_nodes, 
			       t_acfunc activation_function);

void		init_network(t_network *network);

void		free_network(t_network *network);

void		print_network(t_network *network);

void		feed_forward(t_network *network, double *input);

double  	compute_energy(t_network *network, double *expected_output);

#endif /* ! NETWORK_H_ */
