/******************************************************************************
** Project: neural_network
** Filename: prop.c
*******************************************************************************
** Authors: A. BAILLARD & T. BAUDLOT
** Creation date: 14/07/03
** Last modification date:
*******************************************************************************
** Comments:
** 14/07/03	AB	Functions common for propagation algorithms
**				- compute-gradients()
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "network.h"

/**
 ** compute_gradients() computes gradient value for output and hidden layers
 ** parameters:
 **	network: allocated network to train
 **	expected_output: output vector corresponding to input vector
 ** return value:
 **	none
 **/
void	compute_gradients(t_network *network, double *expected_output)
{
  unsigned	i;
  unsigned	k;

  /* init output gradient energy */
  for (i = 0; i < network->nb_output_units; i++)
    network->output_energy[i] = 2 * (network->output_values[i] - expected_output[i]) *
      network->acfunc.func_drv(network->output_activation[i], network->acfunc.lambda);

  /* compute hidden gradient energy */
  for (i = 0; i < network->nb_hidden_units; i++)
    {
      network->hidden_energy[i] = 0;
      for (k = 0; k < network->nb_output_units; k++)
	{
	  network->hidden_energy[i] += 
	    network->output_weights[k][i] * network->output_energy[k];
	}
      network->hidden_energy[i] *= network->acfunc.
	func_drv(network->hidden_activation[i], network->acfunc.lambda);
    }
}
