/******************************************************************************
** Project: neural_network
** Filename: stoch_back_prop.c
*******************************************************************************
** Authors: A. BAILLARD & T. BAUDLOT
** Creation date: 14/07/03
** Last modification date:
*******************************************************************************
** Comments:
** 14/07/03	AB	Functions for stochastic back propagation
**				- compute_gradients()
**				- update_network()
**				- stoch_back_propagation()
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "network.h"

/**
 ** update_network() computes new weights and biases
 ** parameters:
 **	network: allocated network to train
 **	input: input vector
 ** return value:
 **	none
 **/
static void	update_network(t_network *network, double *input)
{
  unsigned	i;
  unsigned	j;

  for (i = 0; i < network->nb_hidden_units; i++)
    {
      for (j = 0; j < network->nb_input_units; j++)
	{
	  network->hidden_weights[i][j] -= 
	    network->epsilon * network->hidden_energy[i] * input[j];
	}
      network->hidden_biases[i] -=
	network->epsilon * network->hidden_energy[i];
    }
  for (i = 0; i < network->nb_output_units; i++)
    {
      for (j = 0; j < network->nb_hidden_units; j++)
	{
	  network->output_weights[i][j] -= network->epsilon * 
	    network->output_energy[i] * network->hidden_values[j];
	}
      network->output_biases[i] -=
	network->epsilon * network->output_energy[i];      
    }
}

/**
 ** stoch_back_propagation() trains the network on a exemple composed of an input and
 ** an expected output - correspond to stochastic algorithm
 ** parameters:
 **	network: allocated network to train
 **	input: input vector
 **	n_input: size of the input vector
 **	expected_output: output vector corresponding to input vector
 **	n_output: size of the expected output vector
 ** return value:
 **	error code
 **/
int		stoch_back_propagation(t_network *network, 
				double *input, unsigned n_input, 
				double *expected_output, unsigned n_output)
{
  if (n_input != network->nb_input_units ||
      n_output != network->nb_output_units)
    {
      printf("Data size do not correspond to network dimensions\n");
      return LIBNN_DATA_SIZE_ERROR;
    }
  
  feed_forward(network, input);

  compute_gradients(network, expected_output);

  update_network(network, input);
  
  compute_energy(network, expected_output);

  return LIBNN_NO_ERROR;
}
