///////////////////////////////////////////////////////////////////////////////
// Project: defectix 
// Filename: simulator.hh
// Directory: src
///////////////////////////////////////////////////////////////////////////////
// Authors: A. BAILLARD
// Creation date: 21/01/04
// Last modification date: 15/03/04
///////////////////////////////////////////////////////////////////////////////
// Comments:
// 21/01/04	Virtual class DefectAdder
// 26/01/04	Add random seed init in constructor
// 28/01/04	Doxygen header
// 29/01/04	Add createSVMOutputFile()
// 26/02/04	Update doxygen header
// 09/03/04	Modify constructor and and two FitsManager
// 10/03/04	Update to simulator.hh and DefectSimulator
// 15/03/04	Clean code
//		Update error management *
//		Move doxygen header to simulator.dox
///////////////////////////////////////////////////////////////////////////////

#ifndef SIMULATOR_H_
# define SIMULATOR_H_

#include <string.h>
#include <stdio.h>
#include "defectix.hh"

using namespace DefectixError;

class	DefectSimulator: public Defectix
{
  /////////////////////////////////////////////////////////////////////////////
  // functions in simulator.cc						     //
  /////////////////////////////////////////////////////////////////////////////
public:
  DefectSimulator(const DefectixOptions::Options& options);

protected:
  const errorCodes	writeAllImageExtensions();

public:
  const errorCodes	exec();
  
  void	addDefectsOnExtension(int nb, int extend);
  void	addDefectsOnMosaic(int nb);
  void	addDefectsPerExtension(int nb);

protected:
  virtual void	createDefect(gsl_matrix_float *src, gsl_matrix_float *mask, int extend) = 0;

protected:
  int		_nbDefects;
  int		_type;
  int		_intensity;
};

#endif /* !SIMULATOR_H_ */
