///////////////////////////////////////////////////////////////////////////////
// Project: defectix 
// Filename: halosimulator.cc
// Directory: src
///////////////////////////////////////////////////////////////////////////////
// Authors: A. BAILLARD
// Creation date: 30/06/04
// Last modification date: 30/06/04
///////////////////////////////////////////////////////////////////////////////
// Comments:
// 30/06/04	Create from astrosimulator.cc
///////////////////////////////////////////////////////////////////////////////

#include "trailsimulator.hh"

using namespace DefectixError;

/////////////////////////////////////////////////////////////////////////////
TrailSimulator::TrailSimulator(const DefectixOptions::Options& options) : 
  DefectSimulator(options)
{
}

// no error can occur in createTrail because everything was checked 
// in addDefectOnExtension()
void		TrailSimulator::createDefect(gsl_matrix_float *src, gsl_matrix_float *mask, int extend)
{
  

  int		row;
  int		col;
  double	coeff[2];
  double	value;
  int		axis[2];
  double	max;

  _imageMgr.extHDU(extend);
  axis[0] = _imageMgr.getAxis(0) / _factor;
  axis[1] = _imageMgr.getAxis(1) / _factor;
  // a = coeff[0] = (y2 - y1) / (x2 - x1)
  coeff[0] = (axis[0] * (rand()/(RAND_MAX + 1.0)) - axis[0] * (rand()/(RAND_MAX + 1.0)))
	       / 
	       (axis[1] * (rand()/(RAND_MAX + 1.0)) - axis[1] * (rand()/(RAND_MAX + 1.0)));
  coeff[1] = (axis[1] * (rand()/(RAND_MAX + 1.0)));
  value = _imagesInfo[extend].max
    / (0.1 * _intensity * (rand()/(RAND_MAX + 1.0)) + 0.95 * _intensity);
  max = _imagesInfo[extend].max / (0.95 * _intensity);

  for (row = 2; row < axis[1] - 2; ++row)
    for (col = 2; col < axis[0] - 2; ++col)
      {
	int ccol = coeff[0] * row + coeff[1];
	int crow = ((float)(col - coeff[1])) / coeff[0];

	if (ccol == col || crow == row)
	  {
	    gsl_matrix_float_set(src, row - 2, col, 0.25 * value);
	    gsl_matrix_float_set(mask, row - 2, col, 0.25 * value / max);
	    gsl_matrix_float_set(src, row, col - 2, 0.25 * value);
	    gsl_matrix_float_set(mask, row, col - 2, 0.25 * value / max);

	    gsl_matrix_float_set(src, row + 2, col, 0.25 * value);
	    gsl_matrix_float_set(mask, row + 2, col, 0.25 * value / max);
	    gsl_matrix_float_set(src, row, col + 2, 0.25 * value);
	    gsl_matrix_float_set(mask, row, col + 2, 0.25 * value / max);

	    gsl_matrix_float_set(src, row - 1, col, 0.25 * value);
	    gsl_matrix_float_set(mask, row - 1, col, 0.25 * value / max);
	    gsl_matrix_float_set(src, row, col - 1, 0.25 * value);
	    gsl_matrix_float_set(mask, row, col - 1, 0.25 * value / max);

	    gsl_matrix_float_set(src, row + 1, col, 0.25 * value);
	    gsl_matrix_float_set(mask, row + 1, col, 0.25 * value / max);
	    gsl_matrix_float_set(src, row, col + 1, 0.25 * value);
	    gsl_matrix_float_set(mask, row, col + 1, 0.25 * value / max);
	  }
      }

  for (row = 1; row < axis[1] - 1; ++row)
    for (col = 1; col < axis[0] - 1; ++col)
      {
	int ccol = coeff[0] * row + coeff[1];
	int crow = ((float)(col - coeff[1])) / coeff[0];

	if (ccol == col || crow == row)
	  {
	    gsl_matrix_float_set(src, row, col, value);
	    gsl_matrix_float_set(mask, row, col, value / max);
	  }
      }
}
