///////////////////////////////////////////////////////////////////////////////
// Project: defectix 
// Filename: trainer.hh
// Directory: src
///////////////////////////////////////////////////////////////////////////////
// Authors: A. BAILLARD
// Creation date: 17/02/04
// Last modification date: 22/10/04
///////////////////////////////////////////////////////////////////////////////
// Comments:
// 17/02/04	Creation
// 24/02/04	Modify createNetworks() to treat the whole MEF
// 10/03/04	Update to trainer.hh and DefectTrainer
// 15/03/04	Move doxygen header to trainer.dox
// 22/10/04	Add writeAllRebuilts()
///////////////////////////////////////////////////////////////////////////////

#ifndef TRAINING_HH_
# define TRAINING_HH_

# include <iostream>
# include <map>
# include <sys/stat.h>
# include "defectix.hh"

extern "C"
{
#  include "network.h"
}

using namespace DefectixError;

class DefectTrainer : public Defectix
{
public:
  /////////////////////////////////////////////////////////////////////////////
  // functions in trainer.cc		       				     //
  /////////////////////////////////////////////////////////////////////////////
  DefectTrainer(const DefectixOptions::Options& options);

public:
  /////////////////////////////////////////////////////////////////////////////
  const errorCodes	exec();

public:
  /////////////////////////////////////////////////////////////////////////////
  const errorCodes	train();

private:
  /////////////////////////////////////////////////////////////////////////////
  errorCodes		createVectors(gsl_matrix_float *input, 
				      gsl_matrix_float *output, 
				      std::map<int, gsl_matrix_float*>& inputSrc,
				      std::map<int, gsl_matrix_float*>& outputSrc,
				      FitsManager& mgr);
  
  /////////////////////////////////////////////////////////////////////////////
  void			createKLBase(gsl_matrix_float	*inputV);

  /////////////////////////////////////////////////////////////////////////////
  const errorCodes	createNetwork(gsl_matrix_float *inputV, 
				      gsl_matrix_float *outputV);


  /////////////////////////////////////////////////////////////////////////////
  const bool		priorBloc(const gsl_vector_float * const vect, const unsigned long extCnt);

  /////////////////////////////////////////////////////////////////////////////
  // attributes				       				     //
  /////////////////////////////////////////////////////////////////////////////
private:  
  t_network	*_l0;
  int		_nbPC;
  int		_nbVectors;
  unsigned	_maxIter;
  double	_energyTreshold;
  std::string	_evecFile;
  std::string	_nnFile;
  bool		_printRebuilt;
  double	_maxPrior;
  double	_minPrior;
  double	_nonPrior;
};

#endif /* !TRAINING_HH_ */
