///////////////////////////////////////////////////////////////////////////////
// Project: defectix 
// Filename: read.cc
// Directory: src
///////////////////////////////////////////////////////////////////////////////
// Authors: A. BAILLARD
// Creation date: 24/03/04
// Last modification date: 15/05/05
///////////////////////////////////////////////////////////////////////////////
// Comments:
// 16/03/04	Create from simulator.cc
//		Contain functions to write images
//			writeAllMaskExtensions()
// 22/10/04	Add writeAllRebuilts()
// 04/05/05	modify writeAllRebuilts() to unread images
// 15/05/05	Change writeAllMasks() to call copyHDU() and setHDUAsImage()
///////////////////////////////////////////////////////////////////////////////

#include "defectix.hh"

///////////////////////////////////////////////////////////////////////////////
const errorCodes	Defectix::writeAllMasks()
{
  char		msg[24];

  if (_maskMgr.getType() == FitsManager::NO_EXTENDED)
    if (_masks[1] != NULL)
      {
	_imageMgr.uniqueHDU();
	_maskMgr.copyHDU(_imageMgr);
	_maskMgr.setHDUAsImage(FLOAT_IMG, 
			       _imageMgr.getNAxes(), 
			       _imageMgr.getAxes(), 
			       _factor);
	
	_maskMgr.writeImage((void **)&(_masks[1]->data));
	sprintf(msg, "Mask extension %d written", 1);
	printMessage(msg);
      }
    else
      return setError("Defectix::writeAllMasks", NO_MASK);

  else
    {
      for (int cnt = 1; cnt <= _imageMgr.getNExtend(); cnt++)
	{
	  if (_masks[cnt] != NULL)
	    {
	      _imageMgr.extHDU(cnt);
	      _maskMgr.copyHDU(_imageMgr);
	      _maskMgr.setHDUAsImage(FLOAT_IMG, 
				     _imageMgr.getNAxes(), 
				     _imageMgr.getAxes(), 
				     _factor);
	      
	      _maskMgr.writeImage((void **)&(_masks[cnt]->data));
	      sprintf(msg, "Mask extension %d written", cnt);
	      printMessage(msg);
	      overwriteMessage();
	    }
	  else
	    return setError("Defectix::writeAllMasks", NO_MASK);
	}
      printMessage(msg);
    }

  return NO_ERROR;
}

const errorCodes	Defectix::writeAllRebuilts(int nbPC)
{
  char			msg[24];
  gsl_matrix_float	*extension;
  
  if ((extension = gsl_matrix_float_alloc(_imageMgr.getAxis(1), _imageMgr.getAxis(0))) == NULL)
    return setError("DefectTrainer::writeAllRebuilts", MALLOC);

  for (int cnt = 1; cnt <= _imageMgr.getNExtend(); cnt++)
    {
      _imageMgr.extHDU(cnt);
      _PCA->computeRebuilt(_images[cnt], extension, nbPC);
      unreadImageExtension(cnt);
      _outputMgr.copyHDU(_imageMgr);
      _outputMgr.setHDUAsImage(FLOAT_IMG, 
			    _imageMgr.getNAxes(), 
			    _imageMgr.getAxes(), 
			    1);

      _outputMgr.writeImage((void **)&(extension->data));
      sprintf(msg, "Rebuilt extension %d written", cnt);
      printMessage(msg);
      overwriteMessage();
    }

  return setError("DefectTrainer::writeAllRebuilts", NO_ERROR);
}
